import serial
from serial.tools import list_ports
import sys

"""
トラ技2023年3月号ダウンロードサービス
「ESP32 のアナログ性能徹底調査」 
"""

# M5Stack ATOM LITE
_USB_VID = 0x0403
_USB_PID = 0x6001

class EspUnit():
    def __init__(self):
        """USB VID, PID から M5Stack ATOM LITEを自動検索して接続"""
        device_lists = list_ports.comports()
        devicename = ''
        for device in device_lists:
            if device.pid == _USB_PID and device.vid == _USB_VID:
                devicename = device.name
                break
        if devicename:
            if sys.platform.startswith("win"):
                self._ser = serial.Serial(devicename, 9600, timeout=0)
            elif sys.platform.startswith("linux"):
                self._ser = serial.Serial('/dev/'+devicename, 9600, timeout=0)
            else:
                self._ser = None
        else:
            self._ser = None

    def __del__(self):
        if self._ser:
            self._ser.close()
            self._ser = None

    def set_dac(self, value: int) -> None:
        """M5Stack ATOM LITE への指令: DAC値設定"""
        if 0<= value <= 255 and self._ser:
            self._ser.write('d{:02x}\r'.format(value).encode())

    def set_adc_range(self, range: int) -> None:
        """M5Stack ATOM LITE への指令: ATT値設定
        0: ATT=0 (0dB)
        1: ATT=1 (2.5dB)
        2: ATT=2 (6dB)
        3: ATT=3 (11dB)
        """
        if range == 0:
            self._ser.write(b'0\r')
        elif range == 1:
            self._ser.write(b'1\r')
        elif range == 2:
            self._ser.write(b'2\r')
        elif range == 3:
            self._ser.write(b'3\r')

    def get_adc(self) -> int:
        """M5Stack ATOM LITE への指令: ADCバイナリ値取得"""
        if self._ser:
            get_str = ''
            self._ser.write(b'a\r')
            while True:
                rd = self._ser.read(1)
                if rd == b'\r':
                    return int(get_str)
                elif rd:
                    get_str += rd.decode()

    def get_mv(self) -> int:
        """M5Stack ATOM LITE への指令: ADC校正電圧値取得"""
        if self._ser:
            get_str = ''
            self._ser.write(b'v\r')
            while True:
                rd = self._ser.read(1)
                if rd == b'\r':
                    return int(get_str)
                elif rd:
                    get_str += rd.decode()
