import time
import pyvisa
import espunit

"""
トラ技2023年3月号ダウンロードサービス
「ESP32 のアナログ性能徹底調査」 
"""

# 事前に調べた計測器のVISAアドレス
GPIB_ADDR = 'GPIB0::2::INSTR'
# 自作の M5Stack 制御クラスオブジェクト espunit.py
esp = espunit.EspUnit()

# リソース管理オブジェクトの生成 (プログラム中唯一)
rm = pyvisa.ResourceManager()
# デジタルマルチメータとの通信路をオープン
meter = rm.open_resource(GPIB_ADDR)
# 計測器からの read() した時、読出し末尾のLF終端文字を除去する設定
meter.read_termination = '\n'
# 計測器のDC電圧測定モード設定
meter.write('CONF:VOLT:DC DEF,DEF')

print('GAIN, DAC, ADC, Calibration[mV], Measurement[V]')
# ATT 0db
# max 1.1V
# recomend 100mV-950mV
esp.set_adc_range(0)
dac_val = 0
while True:
    esp.set_dac(dac_val)
    time.sleep(0.5)
    read_adc = esp.get_adc()
    read_mv = esp.get_mv()
    read_meter = meter.query('READ?')
    print('0, {}, {}, {}, {}'.format(dac_val, read_adc, read_mv, read_meter))
    dac_val += 3        # DACバイナリ値 3づつインクリメント
    if dac_val > 100:   # ADC測定電圧範囲を大きく超過したら止める
        break

# ATT 6db
# max 2.19V
# recomend 150mV-1750mV
esp.set_adc_range(2)
dac_val = 0
while True:
    esp.set_dac(dac_val)
    time.sleep(0.5)
    read_adc = esp.get_adc()
    read_mv = esp.get_mv()
    read_meter = meter.query('READ?')
    print('6, {}, {}, {}, {}'.format(dac_val, read_adc, read_mv, read_meter))
    dac_val += 5        # DACバイナリ値 5づつインクリメント
    if dac_val > 200:   # ADC測定電圧範囲を大きく超過したら止める
        break

# ATT 11db
# max 3.9V
# recomend 150mV-2450mV
esp.set_adc_range(3)
dac_val = 0
while True:
    esp.set_dac(dac_val)
    time.sleep(0.5)
    read_adc = esp.get_adc()
    read_mv = esp.get_mv()
    read_meter = meter.query('READ?')
    print('11, {}, {}, {}, {}'.format(dac_val, read_adc, read_mv, read_meter))
    dac_val += 5      # DACバイナリ値 5づつインクリメント
    if dac_val > 255: # DACフルスケールまで計測
        break

# 計測器のクローズ
meter.close()
# リソース管理オブジェクトのクローズ
rm.close()
del esp
